#!/usr/bin/env python3

import unittest

import conscommon.data
import conscommon.data_model
import conscommon.data_model.mks as mks


class TestDataModel(unittest.TestCase):
    def setUp(self):
        conscommon.data.API_CANDIDATES = ["localhost:8080"]
        self.data_mks = conscommon.data.getMKS()
        self.data_agilent = conscommon.data.getAgilent()
        self.data_mbtemp = conscommon.data.getMBTemp()

    def test_mksDevice(self):
        for device in conscommon.data_model.getDevicesFromBeagles(
            conscommon.data_model.getBeaglesFromList(self.data_mks)
        ):
            self.assertIn(device.enable, [True, False])
            self.assertEqual(device.channels.__len__(), 6)
            self.assertNotEqual(device.info, None)

    def test_mksChannel(self):
        for device in conscommon.data_model.getDevicesFromBeagles(
            conscommon.data_model.getBeaglesFromList(self.data_mks)
        ):
            for channel in device.channels:
                self.assertNotEqual(channel.info, None)
                self.assertIn(channel.name, mks.MKS_CHANNEL_NAMES)
                self.assertIn(
                    channel.info.sensor,
                    [
                        mks.MKS_SENSOR_COLD_CATHODE,
                        mks.MKS_SENSOR_PIRANI,
                        mks.MKS_SENSOR_NOT_USED,
                    ],
                )

    def test_mbtempDevice(self):
        for device in conscommon.data_model.getDevicesFromBeagles(
            conscommon.data_model.getBeaglesFromList(self.data_mbtemp)
        ):
            self.assertIn(device.enable, [True, False])
            self.assertEqual(device.channels.__len__(), 8)

    def test_agilentDevice(self):
        for device in conscommon.data_model.getDevicesFromBeagles(
            conscommon.data_model.getBeaglesFromList(self.data_agilent)
        ):
            self.assertIn(device.enable, [True, False])
            self.assertEqual(device.channels.__len__(), 4)


if __name__ == "__main__":
    unittest.main()



from enum import Enum, unique


@unique
class SheetName(Enum):
    AGILENT = "PVs Agilent 4UHV"
    COUNTING_PRU = "PVs Counting PRU"
    MBTEMP = "PVs MBTemp"
    MKS = "PVs MKS937b"
    SPIXCONV = "PVs SPIxCONV"

    @classmethod
    def has_key(cls, key: str):
        if not key or key.__class__ != str:
            return False
        return key.upper() in cls.__members__.keys()

    @classmethod
    def keys(cls):
        return [k for k in cls.__members__.keys()]

    @classmethod
    def from_key(cls, key: str):
        if not key:
            return None

        for member in cls:
            if member.name == key.upper():
                return member

        return None



import io as _io

import pandas as _pandas
import requests as _requests
import taiga as _taiga

from .. import get_logger

logger = get_logger("Taiga")

_eng_projetos = [
    "ENT001.001 - Projeto de Ondulador Delta para a Linhas Sabiá",
    "ENT001.002 - Projeto para recuperação do Ondulador do UVX (backup Ipê - Sabiá)",
    "ENT001.003 - Projeto de Ondulador Delta de 2.4 metros",
    "ENT001.004 - Projeto de Ondulador Delta de 3.6 metros",
    "ENT001.005 - Projeto de Câmaras de Vácuo para Onduladores Delta 22",
    "ENT001.006 - Projeto de Câmaras de Vácuo para Onduladores Delta 52",
    "ENT001.007 - Projeto de Onduladores Planares",
    "ENT001.008 - Projeto de Ondulador Delta para a Linhas Carnaúba",
    "ENT002.001 - Projeto de amplificadores de potência de RF nacionais",
    "ENT002.002 - Projeto de Torres de RF para operação com da cavidade SC",
    "ENT002.003 - Projeto da Cavidade Harmônica do Sirius",
    "ENT002.004 - Projeto da Cavidade Supercondutora",
    "ENT003.001 - Projeto de ar-condicionado na Sala de Caracterização Magnética",
    "ENT003.002 - Projeto de organização de estoque e armazenagem da ENT",
    "ENT003.003 - Operação e Manutenção de Infraestruturas dos Laboratórios",
    "ENT004.001 - Instalações nos Aceleradores do Sirius",
    "ENT004.002 - Manutenção Preventiva e Corretiva dos Aceleradores do Sirius",
    "ENT005.001 - Projeto do Sistema de Intertravamento dos Aceleradores do Sirius",
    "ENT005.002 - Projeto de Sistema de Correção Rápida para o Sirius (FOFB)",
    "ENT005.003 - Projeto para alinhamento e monitoramento dos aceleradores",
    "ENT005.004 - Projeto do Pré-Font End Máquina",
    "ENT005.005 - Projeto de Sistemas de Monitoramento de Racks para o Sirius",
    "ENT005.006 - Apoio à operação ao Sirius",
    "ENT005.007 - Implementações e testes para operação top-up ",
    "ENT005.008 - Implementações e testes para melhorias da estabilidade do feixe",
    "ENT008.001 - Melhoramentos dos sistemas do LINAC do Sirius",
    "ENT008.002 - Dispositivos pulsados do Sirius ",
    "ENT008.003 - Melhoramentos do Canhão de Elétrons do Sirius",
    "ENT008.004 - Melhoramentos no Booster e Linhas de Transporte",
    "ENT009.001 - Projeto de Planta de Hélio Líquido no Sirius",
    "ENT009.002 - Projeto de sistema de distribuição nítrogênio líquido",
    "ENT011.001 - Projeto de Energização das linhas do faseamento",
    "ENT011.002 - Projeto de Alinhamento de Linhas de Luz XXXXX do Sirius",
    "ENT011.003 - Projeto de Brasagem de componentes para Linhas de Luz do Sirius",
    "ENT011.004 - LCA Laboratório de Ciências Ambientais (Carnaúba S8B5)",
    "ENT011.005 - LCRIO Laboratório de criogenia (Cateretê S8B3)",
    "ENT011.006 - Projeto de Ambiente para CryoLoading  Linha EMA",
    "ENT011.007 - Caracterizações dimensionais e geométricas de detectores e componentes de linhas",
    "ENT011.008 - Sistema de coleta de condensado por vácuo para o Sirius",
    "ENT011.009 - Readequação da fonte para a Linha EMA ",
    "ENT011.010 - Relayout da Sala 908G3 - Sirius",
    "ENT012.001 - Contratação de Empresa de Manutenção Predial do Sirius",
    "ENT012.002 - Projeto de Integração do sistema de incêndio do Sirius do campus",
    "ENT012.003 - Projeto para Renovação de AVCB do Sirius",
    "ENT012.004 - Garantia de obras do Sirius",
    "ENT012.005 - Manutenções Prediais Emergenciais do Sirius",
    "ENT013.001 - Projeto e instalação do sistemas de detecção de incêndio das linhas do Sirius",
    "ENT013.002 - Projeto e instalação de sistemas de exaustão do Sirius",
    "ENT013.003 - Projeto e instalação de sistemas de incêndio especiais (CPD e Carnaúba)",
    "ENT013.004 - Projeto para ampliação do sistema de controle de acessos do Sirius",
    "ENT013.005 - Projeto da Sala do Conselho no Sirius",
    "ENT013.006 - Projeto e Instalação de ar-condicionado na sala de racks do Sirius",
    "ENT013.007 - Projeto de Proteção da Marquise do Sirius",
    "ENT013.008 - Projeto de Revisão do sistema de monitoramento de câmeras do Sirius",
    "ENT013.009 - Projeto do Sistema de Abastecimento de Água (C6) do Sirius",
    "ENT013.010 - Instalações dos sistemas de TI para Salas Video Conferências do Sirius",
    "ENT014.001 - Projeto de recuperação dos Scrapers do Sirius",
    "ENT014.002 - Medição das curvas de excitação dos magnetos do Sirius",
    "ENT014.003 - Melhorias e Caracterização das Fontes do Sirius",
    "ENT014.004 - Projeto e construção de protótipo de conversor AC/DC tiristorizado",
    "ENT014.005 - Projeto da Nova Gaveta de Comando para as Fontes FAC",
    "ENT014.006 - Projeto e construção de protótipo de um novo estágio de saída FAC",
    "ENT014.007 - Projeto e construção de protótipo de fonte de alta corrente e baixa tensão",
    "ENT015.001 - Projeto de Descomissionamento do UVX",
    "ENT016.001 - Projeto de Dipolo Supercondutor (SCBC) para o Sirius - CERN",
    "ENT017.001 - Cursos e Treinamentos de Recursos Humanos",
    "ENT017.002 - Estudos e Desenvolvimentos Internos de Equipes",
    "ENT017.003 - Gestão de Equipes e Recursos Humanos",
    "ENT018.001 - Limpeza e Recuperação de bombas iônicas dos microscópios",
    "ENT018.002 - Desenvolvimento de soluções de automação, controle ou IoT para instalações laboratoriais ",
    "ENT020.001 - Projeto EXA",
    "ENT020.002 - Ciência aberta",
    "ENT020.003 - Atividades Administrativas Internas",
]


def _get_dataframe(url: str):
    return _pandas.read_csv(
        _io.StringIO(_requests.get(url, verify=False).content.decode("utf8"))
    )


_taiga_default_url = "https://10.0.38.59:9011/"

_gas_slug = "grupo-automacao-e-software"
_gas_equipes = ["SM", "TI", "Interlock", "Automação", "Automação dos Delta", "GAS"]
_gas_macro_atividades = [
    "Ansible",
    "Aplicações web no geral",
    "Avaliações de desempenho",
    "Cabeamento de rede",
    "Carrinhos de medição de radiação",
    "Component DB",
    "Computadores da sala de controle",
    "Computadores de trabalho",
    "Conexão da rede do sistema de controle",
    "Conteinerização de aplicações",
    "Contratação de pessoal",
    "DCCTs",
    "Descomissionamento do UVX",
    "Diagnósticos Sirius",
    "Dicionário de PVs",
    "EPICS Archiver",
    "Equipamentos de rede",
    "Espaços físicos para o grupo",
    "Estoque no Sirius",
    "FreeCAD",
    "Gestão das atividades do grupo com o Scrum",
    "HAProxy",
    "Hardware dos sistemas LLRF do booster e anel",
    "IOC das cavidades supercondutoras",
    "IOC das estações de bombeamento de vácuo",
    "IOC das fendas de energia",
    "IOC das gavetas de calibração",
    "IOC das screens do Linac",
    "IOC do sistema LLRF do Linac",
    "IOC do sistema de interlock",
    "IOC dos DCCTs do anel",
    "IOC dos circuladores de RF",
    "IOC dos osciloscópios Keysight",
    "IOC dos power sensors da RF",
    "IOC dos retificadores Regatron",
    "IOCs das torres de amplificadores de RF",
    "IOCs de CLPs Yokogawa",
    "IOCs do sistema de sincronismo",
    "IOCs dos ICTs",
    "IOCs dos detectores de radiação",
    "IOCs dos pulsados",
    "IOCs dos sistemas LLRF do booster e anel",
    "IOCs dos skids",
    "IOCs para o BMS",
    "Integração dos skids ao sistema de controle",
    "Interlock - Linac",
    "Interlock - MPS e PPS",
    "Interlock - MPS",
    "Interlock - PPS",
    "Interlock - RF",
    "Jupyter Lab",
    "Mesa de montagem automatizada dos sub-cassetes dos Delta",
    "Moduladores do Linac",
    "Monitores fluorescentes do Linac",
    "Motores da RF do Linac",
    "Motores do Linac",
    "Motorizados e Acq de Imagens",
    "Motorizados e Acq de imagens",
    "Motorizados",
    "Multímetros",
    "Módulo Python pyDRS",
    "Olog",
    "Onduladores Delta",
    "Onduladores Kyma",
    "Osciloscópios",
    "Outros",
    "Participação em conferências",
    "Portainer",
    "Processos seletivos",
    "Projeto EXA",
    "Projetos com entidades externas",
    "Recuperação da máquina",
    "Redundância da rede do sistema de controle",
    "Repositórios Git",
    "Sala de controle",
    "Servidores da RF",
    "Servidores",
    "Serviço DNS",
    "Serviço NTP",
    "Sistema de arquivamento para o FOFB",
    "Sistema de deposição de NEG",
    "Sistema de notificação de alarmes de PVs",
    "Sistema de sincronismo para as linhas Carcará",
    "Sistema de sincronismo para as linhas de luz",
    "Streak camera",
    "Suporte a outros grupos",
    "Taiga",
    "Telas de supervisório",
    "Torres de amplificadores de RF",
    "Treinamentos",
    "Visualizador web do EPICS Archiver",
    "Zabbix",
]


class TaigaClient:
    def __init__(
        self,
        host: str = _taiga_default_url,
        tls_verify: bool = False,
    ):
        self.api: _taiga.TaigaAPI = _taiga.TaigaAPI(host=host, tls_verify=tls_verify)

    def initialize_project(self, slug: str = _gas_slug):
        project = self.api.projects.get_by_slug(slug)
        self._add_user_story_attributes(project)

    def _add_user_story_attributes(self, project: _taiga.models.Project):
        project.add_user_story_attribute(
            "Projeto",
            description="Projetos válidos da ENG",
            type="dropdown",
            extra=_eng_projetos,
        )
        project.add_user_story_attribute(
            "Macro-atividade",
            description="Macro-atividade da user history",
            type="dropdown",
            extra=_gas_macro_atividades,
        )
        project.add_user_story_attribute(
            "Prioridade Mapeada",
            description="Valor da prioridade (int) utilizada na planilha",
            type="number",
        )
        project.add_user_story_attribute(
            "Variação de pontuação",
            description="Variação de pontuação da atividade durante o sprint",
            type="number",
        )
        project.add_user_story_attribute(
            "Nova?",
            description="Atividade nova no sprint, imprevista",
            type="checkbox",
        )
        project.add_user_story_attribute(
            "Dependências / Condições para a atividade",
            description="Dependências / Condições para a atividade, repersentação textual que será integrada na planilha geral da ENG",
            type="text",
        )
        project.add_user_story_attribute(
            "Nome da Equipe",
            description="Equipe Interna do grupo",
            type="dropdown",
            extra=_gas_equipes,
        )
        project.add_user_story_attribute(
            "Grupo",
            description="Grupos envolvidos",
            type="text",
        )

    def auth(
        self,
        username: str,
        password: str,
    ):
        self.api.auth(username, password)

    def to_excel_table(self, df: _pandas.DataFrame, filename: str = "table.xlsx"):
        # Create a Pandas Excel writer using XlsxWriter as the engine.
        writer = _pandas.ExcelWriter(filename, engine="xlsxwriter")

        # Write the dataframe data to XlsxWriter. Turn off the default header and
        # index and skip one row to allow us to insert a user defined header.
        df.to_excel(
            writer, sheet_name="user_stories", startrow=1, header=False, index=False
        )

        # Get the xlsxwriter workbook and worksheet objects.
        worksheet = writer.sheets["user_stories"]

        # Get the dimensions of the dataframe.
        (max_row, max_col) = df.shape

        # Create a list of column headers, to use in add_table().
        column_settings = [{"header": column} for column in df.columns]

        # Add the Excel table structure. Pandas will add the data.
        worksheet.add_table(0, 0, max_row, max_col - 1, {"columns": column_settings})

        # Make the columns wider for clarity.
        worksheet.set_column(0, max_col - 1, 12)

        # Close the Pandas Excel writer and output the Excel file.
        writer.save()

    def get_stories_df(self, slug: str = _gas_slug):
        project = self.api.projects.get_by_slug(slug)
        userstories_csv_uuid = project.userstories_csv_uuid
        if not userstories_csv_uuid:
            msg = f"Project '{slug}' does not have a user story report enabled. Please enable it using the Taiga front-end before continuing."
            raise Exception(msg)

        userstories_csv_url = (
            f"{self.api.host}/api/v1/userstories/csv?uuid={userstories_csv_uuid}"
        )

        logger.info(f"Using User Story Report '{userstories_csv_url}'")
        stories_df = _get_dataframe(userstories_csv_url)

        filtered_df = stories_df.rename(
            columns={
                "assigned_users_full_name": "Membros Atribuidos",
                "description": "Descrição",
                "id": "Taiga Story ID",
                "sprint": "Sprint",
                "status": "Status",
                "subject": "Atividades",
                "tags": "Taiga Tags",
                "tasks": "Taiga Tasks ID",
                "total-points": "Número de Pontos",
            }
        )

        columns = [
            "Projeto",
            "Prioridade Mapeada",
            "Macro-atividade",
            "Atividades",
            "Status",
            "Número de Pontos",
            "Grupo",
            "Nome da Equipe",
            "Descrição",
            "Dependências / Condições para a atividade",
            "Variação de pontuação",
            "Nova?",
            "Membros Atribuidos",
            "Sprint",
            "Taiga Story ID",
            "Taiga Tasks ID",
            "Taiga Tags",
        ]
        output_df = filtered_df[columns]
        output_df["Sprint"].fillna("Backlog", inplace=True)
        output_df["Prioridade Mapeada"].fillna(500, inplace=True)
        output_df["Grupo"].fillna("GAS", inplace=True)
        output_df["Nome da Equipe"].fillna("GAS", inplace=True)

        output_df["Nova?"].fillna("Não", inplace=True)
        mask = output_df["Nova?"] == True  # noqa: E712
        output_df.loc[mask, "Nova?"] = "Sim"
        mask = output_df["Nova?"] == False  # noqa: E712
        output_df.loc[mask, "Nova?"] = "Não"

        output_df.fillna("", inplace=True)
        return output_df



import abc as _abc
import typing as _typing

import requests as _requests

from .types import (
    ArchiverDisconnectedPV,
    ArchiverDroppedByTypeChangePV,
    ArchiverGenericReponse,
    ArchiverPausedPV,
    ArchiverSamplingMethod,
    ArchiverStatusPV,
    _make_archiver_disconnected_pv,
    _make_archiver_paused_pv,
    _make_archiver_status_pv,
)

_default_base_url = "https://10.0.38.42"


class ArchiverLoginException(Exception):
    def __init__(self, *args: object) -> None:
        super().__init__(*args)


class ArchiverMGMTClient(_abc.ABC):
    @_abc.abstractmethod
    def get_all_pvs(self, pv_name: str, regex: str, limit: int):
        """
        /getAllPVs - Get all the PVs in the cluster. Note this call can return millions of PVs
        pv An optional argument that can contain a GLOB wildcard. We will return PVs that match this GLOB. For example, if pv=KLYS*, the server will return all PVs that start with the string KLYS. If both pv and regex are unspecified, we match against all PVs.
        regex An optional argument that can contain a Java regex wildcard. We will return PVs that match this regex. For example, if pv=KLYS*, the server will return all PVs that start with the string KLYS.
        limit An optional argument that specifies the number of matched PV's that are retured. If unspecified, we return 500 PV names. To get all the PV names, (potentially in the millions), set limit to –1.
        """

    @_abc.abstractclassmethod
    def pause_pv(self, pv_name: str):
        """
        /pauseArchivingPV - Pause archiving the specified PV.
        This also tears down the CA channel for this PV.

        pv : The name of the pv.

        You can also pass in GLOB wildcards here and multiple PVs as a comma separated list.
        If you have more PVs that can fit in a GET, send the pv's as
        a CSV pv=pv1,pv2,pv3 as the body of a POST.
        """


class ArchiverClient(ArchiverMGMTClient):
    def __init__(
        self,
        base_url: str = _default_base_url,
        username: _typing.Optional[str] = None,
        password: _typing.Optional[str] = None,
    ) -> None:

        self._mgmt_url = f"{base_url}/mgmt/bpl"
        self._retrieval_url = f"{base_url}/retrieval/bpl"
        self._retrieval_data_url = f"{base_url}/retrieval/data"
        self._authenticated = False

        self._session: _requests.Session = _requests.Session()
        self._session.verify = False
        if username and password:
            self.login(username, password)

    def get_all_pvs(self, pv_name: str, regex: str, limit: int):
        raise NotImplementedError()

    @property
    def authenticated(self):
        return self._authenticated

    def login(self, username: str, password: str) -> ArchiverGenericReponse:
        data = {"username": username, "password": password}

        response = self._session.post(
            f"{self._mgmt_url}/login", data=data, verify=False
        )
        self._authenticated = (
            response.status_code == 200 and "authenticated" in response.text
        )
        if not self.authenticated:
            raise ArchiverLoginException(f"failed to authenticat user {username}")

        response_json = None
        try:
            response_json = response.json()
        except ValueError:
            response_json = None

        return ArchiverGenericReponse(
            success=self.authenticated,
            status_code=response.status_code,
            text=response.text,
            json=response_json,
        )

    def logout(self):
        raise NotImplementedError()

    def resume_pv(self, pv_name: str) -> ArchiverGenericReponse:
        raise NotImplementedError()

    def delete_pv(self, pv_name: str) -> ArchiverGenericReponse:
        raise NotImplementedError()

    def pause_pv(self, pv_name: str) -> ArchiverGenericReponse:
        if not self.authenticated:
            raise ArchiverLoginException("operation requires authentication")

        if not pv_name:
            raise ValueError()

        response = self._session.get(f"{self._mgmt_url}/pauseArchivingPV?pv={pv_name}")
        success = bool(
            response.status_code == 200
            and (f"Successfully paused the archiving of PV {pv_name}")
            or (f"PV {pv_name} is already paused") in response.text
        )
        response_json = None
        try:
            response_json = response.json()
        except ValueError:
            response_json = None

        return ArchiverGenericReponse(
            success=success,
            status_code=response.status_code,
            text=response.text,
            json=response_json,
        )

    def change_pv_archival_parameters(
        self, pv: str, sampling_method: ArchiverSamplingMethod, sampling_period: float
    ):
        """
        /changeArchivalParameters - Change the archival parameters for a PV.
        pv: The name of the pv.
        samplingperiod: The new sampling period in seconds.
        samplingmethod: The new sampling method For now, this is one of SCAN or MONITOR.
        """
        if sampling_method < 0.01:
            raise AttributeError(f"'sampling_period' {sampling_period} invalue range")
        if (
            type(sampling_method) != str
            and type(sampling_method) != ArchiverSamplingMethod
        ):
            raise AttributeError(f"'sampling_method' {sampling_method} invalid type")
        if (
            sampling_method != ArchiverSamplingMethod.SCAN
            and sampling_method != ArchiverSamplingMethod.MONITOR
        ):
            raise AttributeError(f"'sampling_method' {sampling_method} invalid option")

        # sampling_method_value = sampling_method.value

        # /changeArchivalParameters
        raise NotImplementedError()

    def get_pv_details(self, pv_name: str):
        raise NotImplementedError()

    def get_currently_disconnected_pvs(self) -> _typing.List[ArchiverDisconnectedPV]:
        pvs_json = self._session.get(
            f"{self._mgmt_url}/getCurrentlyDisconnectedPVs", verify=False
        ).json()
        return [_make_archiver_disconnected_pv(**pv) for pv in pvs_json]

    def get_paused_pvs_report(self) -> _typing.List[ArchiverPausedPV]:
        return [
            _make_archiver_paused_pv(**pv)
            for pv in self._session.get(
                f"{self._mgmt_url}/getPausedPVsReport", verify=False
            ).json()
        ]

    def get_matching_pvs(self, search: str, limit: int = 500) -> _typing.List[str]:
        return [
            pv
            for pv in self._session.get(
                f"{self._retrieval_url}/getMatchingPVs",
                params={"pv": search, "limit": limit},
                verify=False,
            ).json()
        ]

    def get_pv_status(self, search: str, reporttype: str = "short") -> ArchiverStatusPV:
        return [
            _make_archiver_status_pv(**pv)
            for pv in self._session.get(
                f"{self._mgmt_url}/getPVStatus",
                params={"pv": search, "reporttype": reporttype},
                verify=False,
            ).json()
        ]

    def get_pvs_by_dropped_events_type_change(
        self, limit: int = 200
    ) -> _typing.List[ArchiverDroppedByTypeChangePV]:
        return [
            ArchiverDroppedByTypeChangePV(
                pv_name=pv["pvName"], events_dropped=pv["eventsDropped"]
            )
            for pv in self._session.get(
                f"{self._mgmt_url}/getPVsByDroppedEventsTypeChange",
                params={limit: limit},
            ).json()
        ]

    def change_type_for_pv(self):
        raise NotImplementedError()


def getPVsByDroppedEventsTypeChange(
    base_url: str = _default_base_url,
) -> _typing.List[ArchiverDroppedByTypeChangePV]:
    return ArchiverClient(base_url=base_url).get_pvs_by_dropped_events_type_change()


def getCurrentlyDisconnectedPVs(
    base_url: str = _default_base_url,
) -> _typing.List[ArchiverDisconnectedPV]:
    return ArchiverClient(base_url=base_url).get_currently_disconnected_pvs()


def getPausedPVsReport(
    base_url: str = _default_base_url,
) -> _typing.List[ArchiverPausedPV]:
    return ArchiverClient(base_url=base_url).get_paused_pvs_report()


def getMatchingPVs(search: str, base_url: str = _default_base_url) -> _typing.List[str]:
    return ArchiverClient(base_url=base_url).get_matching_pvs(search=search)


def getPVStatus(search: str, base_url: str = _default_base_url):
    return ArchiverClient(base_url=base_url).get_pv_status(search=search)



import enum as _enum
import typing as _typing


class ArchiverSamplingMethod(str, _enum.Enum):
    MONITOR = "MONITOR"
    SCAN = "SCAN"


class ArchiverDisconnectedPV(_typing.NamedTuple):
    command_thread_id: str
    connection_lost_at: str
    extras: dict
    host_name: str
    instance: str
    internal_state: str
    last_known_event: str
    no_connection_as_of_epoch_secs: str
    pv_name: str


def _make_archiver_disconnected_pv(
    commandThreadID,
    connectionLostAt,
    hostName,
    instance,
    internalState,
    lastKnownEvent,
    noConnectionAsOfEpochSecs,
    pvName,
    **extras,
) -> ArchiverDisconnectedPV:
    return ArchiverDisconnectedPV(
        command_thread_id=commandThreadID,
        connection_lost_at=connectionLostAt,
        extras=extras,
        host_name=hostName,
        instance=instance,
        internal_state=internalState,
        last_known_event=lastKnownEvent,
        no_connection_as_of_epoch_secs=noConnectionAsOfEpochSecs,
        pv_name=pvName,
    )


class ArchiverPausedPV(_typing.NamedTuple):
    pv_name: str
    instance: str
    modification_time: str


def _make_archiver_paused_pv(pvName, instance, modificationTime, **extras):
    return ArchiverPausedPV(
        pv_name=pvName, instance=instance, modification_time=modificationTime
    )


class ArchiverStatusPVExtras(_typing.NamedTuple):
    appliance: str
    connection_first_established: str
    connection_last_restablished: str
    connection_loss_regain_count: str
    connection_state: str
    is_monitored: str
    last_event: str
    last_rotate_logs: str
    pv_name_only: str
    sampling_period: str


class ArchiverStatusPV(_typing.NamedTuple):
    pv_name: str
    status: str
    extras: _typing.Optional[ArchiverStatusPVExtras]


class ArchiverDroppedByTypeChangePV(_typing.NamedTuple):
    events_dropped: str
    pv_name: str


class ArchiverGenericReponse(_typing.NamedTuple):
    success: bool
    status_code: int
    text: str
    json: _typing.Optional[_typing.Union[list, dict]]


def _make_archiver_status_pv(pvName: str, status: str, **extras):
    return ArchiverStatusPV(
        pv_name=pvName,
        status=status,
        extras=None
        if not extras
        else ArchiverStatusPVExtras(
            appliance=extras["appliance"],
            connection_first_established=extras["connectionFirstEstablished"],
            connection_last_restablished=extras["connectionLastRestablished"],
            connection_loss_regain_count=extras["connectionLossRegainCount"],
            connection_state=extras["connectionState"],
            is_monitored=extras["isMonitored"],
            last_event=extras["lastEvent"],
            last_rotate_logs=extras["lastRotateLogs"],
            pv_name_only=extras["pvNameOnly"],
            sampling_period=extras["samplingPeriod"],
        ),
    )



#!/usr/bin/env python

import csv
import io
import json

import requests

from flask import Flask, Response, request

BASE_API_URL = "https://10.0.38.59:9011/api/v1"
BASE_TASKS = f"{BASE_API_URL}/tasks/csv"
BASE_USER_STORY = f"{BASE_API_URL}/userstories/csv"

app = Flask(__name__)


def load_data(url, params):
    return io.StringIO(
        requests.get(url, params=params, verify=False).content.decode("utf8")
    )


@app.route("/taiga/tasks", methods=["GET"])
def tasks():
    uuid = request.args.get("uuid")
    if not uuid:
        return "Missing param uuid", 400

    csv_f = load_data(BASE_TASKS, params={"uuid": uuid})
    return Response(
        json.dumps(list(csv.DictReader(csv_f))), mimetype="application/json"
    )


@app.route("/taiga/stories", methods=["GET"])
def stories():
    uuid = request.args.get("uuid")
    if not uuid:
        return "Missing param uuid", 400

    csv_f = load_data(BASE_USER_STORY, params={"uuid": uuid})
    return Response(
        json.dumps(list(csv.DictReader(csv_f))), mimetype="application/json"
    )


if __name__ == "__main__":
    app.run(debug=True)
